//
//  PixelView.h
//  Pixellation
//
//  Created by Shon Frazier on 3/1/12.
//  Copyright (c) 2012 Praxxium. All rights reserved.
//

#import <AppKit/AppKit.h>
#import "NSImage+CGImageConversion.h"
#import "AttributeBezierPath.h"


typedef enum {
	PVSampleMethodAverageOfBlock = 1,
	PVSampleMethodSinglePixel = 2
} PVSampleMethod;

typedef enum {
	PVPixelShapeSquare = 0, //default
	PVPixelShapeRoundedRect = 1, //requires corner diameter
	PVPixelShapeCircle = 2,
	PVPixelShapeBezierPath = 3 //requires a bezier path, which should exist within a single unit in quadrant one, [0, 1]
} PVPixelShape;

typedef enum {
	PVBackgroundStyleNone = 0,
	PVBackgroundStyleSolidColor = 1,
	PVBackgroundStyleImage = 2, //tiles
} PVBackgroundStyle;

@interface PixelView : NSView <NSDraggingDestination> {
	NSMutableSet *abpaths;
	NSBitmapImageRep *sourceImageRep;
	
	// sample attributes
	NSUInteger sampleRate; //how often to sample the source image
	PVSampleMethod sampleMethod;
	NSUInteger whichPixel; // if sampleMethod==PVSampleMethodSinglePixel
	
	// output attributes
	CGFloat pixelSize; //always square
	CGFloat padding; //equal on all sides
	PVPixelShape pixelShape; //not yet implemented
	
	PVBackgroundStyle backgroundStyle; // for the background drawn between pixels
	NSColor *pixelBackgroundColor;
	NSImage *pixelBackgroundImage;
	
	NSRect finalRect;
	
	NSDragOperation dragOp;
	NSImage *draggedImage;
	
	CGFloat gestureMagnification;
	CGFloat gestureRotation;
}

- (id) initWithFrame:(NSRect)frameRect;

@property (retain) IBOutlet NSBitmapImageRep *sourceImageRep;

@property (assign) NSUInteger sampleRate;
@property (assign) PVSampleMethod sampleMethod;
@property (assign) NSUInteger whichPixel;

@property (assign) CGFloat pixelSize;
@property (assign) CGFloat padding;
@property (assign) PVPixelShape pixelShape;

@property (assign) PVBackgroundStyle backgroundStyle;
@property (strong) NSColor *pixelBackgroundColor;
@property (strong) NSImage *pixelBackgroundImage;

@end


