//
//  PixelView.m
//  Pixellation
//
//  Created by Shon Frazier on 3/1/12.
//  Copyright (c) 2012 Praxxium. All rights reserved.
//

#import "PixelView.h"


// NOTE: This is not exactly reusable outside this file. It's completely dependent on the settings 
// used to make the bitmap in NSImage(CGImageConversion)-bitmap.
static NSColor *NSColorFromPixel(NSUInteger pixel[8]) {
	CGFloat redf, greenf, bluef, alphaf;
	NSUInteger redi, greeni, bluei, alphai;
	
	alphai = pixel[0];
	redi = pixel[1];
	greeni = pixel[2];
	bluei = pixel[3];
	
	redf = ((CGFloat)redi)/255.;
	greenf = ((CGFloat)greeni)/255.;
	bluef = ((CGFloat)bluei)/255.;
	alphaf = ((CGFloat)alphai)/255.;
	
	return [NSColor colorWithCalibratedRed:redf green:greenf blue:bluef alpha:alphaf];
}

static NSImage *NSImageFromCGImage(CGImageRef cgi) {
	CGRect rect;
	rect.size.width = CGImageGetWidth(cgi);
	rect.size.height = CGImageGetWidth(cgi);
	rect.origin = NSMakePoint(0., 0.);
	
	NSImage* image = [[NSImage alloc] initWithSize:rect.size];
	[image lockFocus];
	CGContextDrawImage([[NSGraphicsContext currentContext] graphicsPort], *(CGRect*)&rect, cgi);
	[image unlockFocus];
	
	return image;
}

static NSBitmapImageRep *NSBitmapImageRepFromCGImage(CGImageRef cgimage) {
	size_t width = CGImageGetWidth(cgimage);
	size_t height = CGImageGetHeight(cgimage);
	size_t bitsPerComponent = CGImageGetBitsPerComponent(cgimage);
	size_t bytesPerRow = CGImageGetBytesPerRow(cgimage);
	CGColorSpaceRef space = CGImageGetColorSpace(cgimage);
	CGBitmapInfo bitmapInfo = CGImageGetBitmapInfo(cgimage);
	
	CGRect rect;
	rect.size.width = width;
	rect.size.height = height;
	rect.origin.x = 0.;
	rect.origin.y = 0.;
	
	CGContextRef bitmapCtx = CGBitmapContextCreate(NULL, width, height, bitsPerComponent, bytesPerRow, space, bitmapInfo);
	CGContextDrawImage(bitmapCtx, rect, cgimage);
	
	return nil;
}



@implementation PixelView

@synthesize  sourceImageRep, sampleRate, sampleMethod, pixelBackgroundImage, pixelBackgroundColor, backgroundStyle, whichPixel, pixelShape, pixelSize, padding;

- (id) initWithFrame:(NSRect)frameRect {
	if (self = [super initWithFrame:frameRect]) {
		[self registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, nil]];

		self.sourceImageRep = nil;
		
		// sample attributes
		self.sampleRate = 1;
		self.sampleMethod = PVSampleMethodSinglePixel;
		self.whichPixel = 0;
		
		// output attributes
		self.pixelSize = 12.0;
		self.padding = 0.0; //equal on all sides
		//self.pixelShape; //not yet implemented
		
		self.backgroundStyle = PVBackgroundStyleNone; // for the background drawn between pixels
		self.pixelBackgroundColor = [[NSColor whiteColor] colorWithAlphaComponent:0.0];
		self.pixelBackgroundImage = nil;
		
		dragOp = NSDragOperationNone;
		finalRect = NSZeroRect;
		gestureMagnification = 1.0;
		gestureRotation = 0.0;
		
		//self.pixelBackgroundImage = [[NSImage alloc] initByReferencingURL:[NSURL URLWithString:@"http://subtlepatterns.com/patterns/random_grey_variations.png"]];
		//self.backgroundStyle = PVBackgroundStyleImage;
		
		[self addObserver:self forKeyPath:@"sourceImageRep" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"sampleRate" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"sampleMethod" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"pixelBackgroundImage" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"pixelBackgroundColor" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"backgroundSryle" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"whichPixel" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"pixelShape" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"pixelSize" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"padding" options:NSKeyValueObservingOptionNew context:NULL];
	}
	
	return self;
}

#ifdef ALLOW_GESTURES
- (void) magnifyWithEvent:(NSEvent *)event {
	//DLog(@"entered");
	gestureMagnification += event.magnification;
	self.needsDisplay = YES;
}

- (void) rotateWithEvent:(NSEvent *)event {
	gestureRotation += (event.rotation * (self.isFlipped?-1:1));
	self.needsDisplay = YES;
}
#endif

- (void) observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	//DLog(@"%@ changed: %@", keyPath, change);
	abpaths = nil;
	self.needsDisplay = YES;
}

- (BOOL) isFlipped { return YES; }

- (void) drawRect:(NSRect)dirtyRect {
	//DLog(@"entered");
	if ((sourceImageRep) && ((!abpaths)) ) {
		abpaths = [NSMutableSet set];
		
		NSUInteger pixel[4]; memset(pixel, 0, sizeof(pixel));
		NSUInteger x, y;
		NSRect currentRectangle;
		currentRectangle.size.height = 
		currentRectangle.size.width = pixelSize;
		
		CGFloat finalWidth = padding + (pixelSize + padding)*(sourceImageRep.pixelsWide/sampleRate);
		CGFloat finalHeight = padding + (pixelSize + padding)*(sourceImageRep.pixelsHigh/sampleRate);
		
		finalRect = NSMakeRect(0.0, 0.0, finalWidth, finalHeight);
		
		AttributedBezierPath *abpath;
		for(y=0; (y*sampleRate)<sourceImageRep.pixelsHigh; y++) {
			currentRectangle.origin.y = ((CGFloat)(y))*(currentRectangle.size.height + padding) + padding;
			
			for(x=0; (x*sampleRate)<sourceImageRep.pixelsWide; x++) {
				currentRectangle.origin.x = ((CGFloat)(x))*(currentRectangle.size.width + padding) + padding;
				
				[sourceImageRep getPixel:pixel atX:(x*sampleRate) y:(y*sampleRate)];
				
				abpath = [AttributedBezierPath attributedBezierPath];
				[abpath.bezierPath appendBezierPathWithRect:currentRectangle];
				abpath.fillColor = NSColorFromPixel(pixel);
				abpath.shouldFill = YES;
				
				[abpaths addObject:abpath];
			}
		}
	}
	
#ifdef ALLOW_GESTURES
	NSAffineTransform *t = [NSAffineTransform transform];
	[t scaleBy:gestureMagnification];
	[t translateXBy:finalRect.size.width/2 yBy:finalRect.size.height/2];
	[t rotateByDegrees:gestureRotation];
	[t translateXBy:-finalRect.size.width/2 yBy:-finalRect.size.height/2];
	[t concat];
#endif

	if(sourceImageRep != nil)  {
		// draw background
		switch (backgroundStyle) {
			case PVBackgroundStyleSolidColor:
				if(pixelBackgroundColor!=nil) {
					NSBezierPath *bg = [NSBezierPath bezierPath];
					[bg appendBezierPathWithRect:self.bounds];
					[pixelBackgroundColor setFill];
					[bg fill];
				}
				break;
			case PVBackgroundStyleImage:
				if(pixelBackgroundImage!=nil) {
					NSColor *bg = [NSColor colorWithPatternImage:pixelBackgroundImage];
					[bg setFill];
					NSRectFill(finalRect);
				}
				break;
			default:
				break;
		}
	}
	
	for(AttributedBezierPath *abp in abpaths) {
		[abp draw];
	}
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
	NSPasteboard *pboard = [sender draggingPasteboard];
	NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
	
	if([files count]!=1) { dragOp = NSDragOperationNone; }
	else {
		NSString *fileName = [files objectAtIndex:0];
		
		NSImage *img = [[NSImage alloc] initByReferencingFile:fileName];
		if(!img) dragOp = NSDragOperationNone;
		else {
			dragOp = NSDragOperationCopy;
		}
	}
	
	return dragOp;
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender {
	return dragOp;
}

- (void) draggingEnded:(id<NSDraggingInfo>)sender {
	
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender {
	NSPasteboard *pboard = [sender draggingPasteboard];
	NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
	NSString *fileName = [files objectAtIndex:0];
	
	draggedImage = [[NSImage alloc] initByReferencingFile:fileName];
	
	return YES;
}

- (BOOL) performDragOperation:(id<NSDraggingInfo>)sender {
	self.sourceImageRep = [draggedImage bitmap];
	return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender {
}

@end

